@@ -1,8 +1,3 @@
-import math
-import torch
-from time import time
-import numpy as np
-
 class KMeans:
   '''
   Kmeans clustering algorithm implemented with PyTorch
@@ -168,40 +163,41 @@ class KMeans:
       else:
         x = X
       closest = self.max_sim(a=x, b=self.centroids)[1]
-      if self.minibatch is not None:
-        matched_clusters, counts = closest.unique(return_counts=True)
-      else:
-        matched_clusters, counts = closest.unique(return_counts=True)
+      matched_clusters, counts = closest.unique(return_counts=True)
 
       c_grad = torch.zeros_like(self.centroids)
       if self._loop:
         for j, count in zip(matched_clusters, counts):
           c_grad[j] = x[closest==j].sum(dim=0) / count
       else:
-        # expanded_closest = closest[None].expand(len(matched_clusters), -1)
-        # mask = (expanded_closest==matched_clusters[:, None]).float()
-        # prod = mask @ x / mask.sum(1)[:, None]
-        # c_grad[matched_clusters] = prod
-
-        if x.dtype == torch.float:
-          expected = closest.numel() * len(matched_clusters) * 5 # bool+float
-        elif x.dtype == torch.half:
-          expected = closest.numel() * len(matched_clusters) * 3 # bool+half
-        if device == 'cpu':
-          ratio = 1
+        if self.minibatch is None:
+          expanded_closest = closest[None].expand(self.n_clusters, -1)
+          mask = (expanded_closest==torch.arange(self.n_clusters, device=device)[:, None]).float()
+          c_grad = mask @ x / mask.sum(-1)[..., :, None]
         else:
-          ratio = math.ceil(expected / self.remaining_memory() )
-        # ratio = 1
-        subbatch_size = math.ceil(len(matched_clusters)/ratio)
-        for j in range(ratio):
-          if j*subbatch_size >= batch_size:
-            continue
-          sub_matched_clusters = matched_clusters[j*subbatch_size: (j+1)*subbatch_size]
-          sub_expanded_closest = closest[None].expand(len(sub_matched_clusters), -1)
-          sub_mask = (sub_expanded_closest==sub_matched_clusters[:, None]).to(x.dtype)
-          sub_prod = sub_mask @ x / sub_mask.sum(1)[:, None]
-          c_grad[sub_matched_clusters] = sub_prod
+          expanded_closest = closest[None].expand(len(matched_clusters), -1)
+          mask = (expanded_closest==matched_clusters[:, None]).float()
+          c_grad[matched_clusters] = mask @ x / mask.sum(-1)[..., :, None]
 
+
+        # if x.dtype == torch.float:
+        #   expected = closest.numel() * len(matched_clusters) * 5 # bool+float
+        # elif x.dtype == torch.half:
+        #   expected = closest.numel() * len(matched_clusters) * 3 # bool+half
+        # if device == 'cpu':
+        #   ratio = 1
+        # else:
+        #   ratio = math.ceil(expected / self.remaining_memory() )
+        # # ratio = 1
+        # subbatch_size = math.ceil(len(matched_clusters)/ratio)
+        # for j in range(ratio):
+        #   if j*subbatch_size >= batch_size:
+        #     continue
+        #   sub_matched_clusters = matched_clusters[j*subbatch_size: (j+1)*subbatch_size]
+        #   sub_expanded_closest = closest[None].expand(len(sub_matched_clusters), -1)
+        #   sub_mask = (sub_expanded_closest==sub_matched_clusters[:, None]).to(x.dtype)
+        #   sub_prod = sub_mask @ x / sub_mask.sum(1)[:, None]
+        #   c_grad[sub_matched_clusters] = sub_prod
       error = (c_grad - self.centroids).pow(2).sum()
       if self.minibatch is not None:
         lr = 1/num_points_in_clusters[:,None] * 0.9 + 0.1
@@ -240,7 +236,7 @@ class KMeans:
       Return:
       labels: torch.Tensor, shape: [n_samples]
     """
-    return self.max_sim(a=X, b=self.centroids, device=x.device.type )[1]
+    return self.max_sim(a=X, b=self.centroids)[1]
 
   def fit(self, X, centroids=None):
     """
